@@ -10,13 +10,14 @@
 from __future__ import print_function
 import requests
 import json
+import os
 from watson_machine_learning_client.utils import INSTANCE_DETAILS_TYPE, RUNTIME_SPEC_DETAILS_TYPE, MODEL_DETAILS_TYPE, LIBRARY_DETAILS_TYPE, FUNCTION_DETAILS_TYPE, STR_TYPE, STR_TYPE_NAME, get_type_of_details, docstring_parameter, str_type_conv, print_text_header_h2
 from watson_machine_learning_client.wml_client_error import WMLClientError
 from watson_machine_learning_client.href_definitions import is_uid
 from watson_machine_learning_client.wml_resource import WMLResource
 from watson_machine_learning_client.metanames import RuntimeMetaNames, LibraryMetaNames
-from repository_v3.mlrepositoryartifact import MLRepositoryArtifact
-from repository_v3.mlrepository import MetaProps, MetaNames
+from watson_machine_learning_client.libs.repo.mlrepositoryartifact import MLRepositoryArtifact
+from watson_machine_learning_client.libs.repo.mlrepository import MetaProps, MetaNames
 
 
 def LibraryDefinition(name, version, filepath, description=None, platform=None):
@@ -47,10 +48,12 @@ class Runtimes(WMLResource):
     """
     def __init__(self, client):
         WMLResource.__init__(self, __name__, client)
-        Runtimes._validate_type(client.service_instance.details, u'instance_details', dict, True)
-        Runtimes._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
+        if not client.ICP:
+            Runtimes._validate_type(client.service_instance.details, u'instance_details', dict, True)
+            Runtimes._validate_type_of_details(client.service_instance.details, INSTANCE_DETAILS_TYPE)
         self.ConfigurationMetaNames = RuntimeMetaNames()
         self.LibraryMetaNames = LibraryMetaNames()
+        self._ICP = client.ICP
 
     def _create_library_from_definition(self, definition, runtime_definition):
         self._validate_meta_prop(definition, 'name', STR_TYPE, True)
@@ -90,14 +93,14 @@ class Runtimes(WMLResource):
             :returns: details of created custom library
             :rtype: dict
 
-            A way you might use me is:
+            **Example**:
 
              >>> library_details = client.runtimes.store_library({
-                        self.client.runtimes.LibraryMetaNames.NAME: "libraries_custom",
-                        self.client.runtimes.LibraryMetaNames.DESCRIPTION: "custom libraries for scoring",
-                        self.client.runtimes.LibraryMetaNames.FILEPATH: custom_library_path,
-                        self.client.runtimes.LibraryMetaNames.VERSION: "1.0",
-                        self.client.runtimes.LibraryMetaNames.PLATFORM: {"name": "python", "versions": ["3.5"]}
+                        client.runtimes.LibraryMetaNames.NAME: "libraries_custom",
+                        client.runtimes.LibraryMetaNames.DESCRIPTION: "custom libraries for scoring",
+                        client.runtimes.LibraryMetaNames.FILEPATH: custom_library_path,
+                        client.runtimes.LibraryMetaNames.VERSION: "1.0",
+                        client.runtimes.LibraryMetaNames.PLATFORM: {"name": "python", "versions": ["3.5"]}
                     })
          """
         self.LibraryMetaNames._validate(meta_props)
@@ -155,7 +158,7 @@ class Runtimes(WMLResource):
 
             :param meta_props: dictionary with parameters describing runtime spec. To see available meta names use:
 
-               >>> client.runtimes.DefinitionMetaNames.get()
+               >>> client.runtimes.ConfigurationMetaNames.get()
             :type meta_props: dict
 
             :returns: details of created runtime
@@ -222,15 +225,15 @@ class Runtimes(WMLResource):
            Get information about your runtime(s).
 
            :param runtime_uid:  Runtime UID (optional)
-           :type runtime_uid: {str_type}
+           :type runtime_uid: str
 
            :param limit: limit number of fetched records (optional)
            :type limit: int
 
            :returns: metadata of runtime(s)
-           :rtype: dict
+           :rtype: dict (if uid is not None) or {"resources": [dict]} (if uid is None)
 
-           A way you might use me is:
+           **Example**:
 
             >>> runtime_details = client.runtimes.get_details(runtime_uid)
             >>> runtime_details = client.runtimes.get_details(runtime_uid=runtime_uid)
@@ -239,8 +242,8 @@ class Runtimes(WMLResource):
         runtime_uid = str_type_conv(runtime_uid)
         Runtimes._validate_type(runtime_uid, u'runtime_uid', STR_TYPE, False)
 
-        if runtime_uid is not None and not is_uid(runtime_uid):
-            raise WMLClientError(u'\'runtime_uid\' is not an uid: \'{}\''.format(runtime_uid))
+        # if runtime_uid is not None and not is_uid(runtime_uid):
+        #     raise WMLClientError(u'\'runtime_uid\' is not an uid: \'{}\''.format(runtime_uid))
 
         url = self._href_definitions.get_runtimes_href()
 
@@ -251,15 +254,15 @@ class Runtimes(WMLResource):
            Get information about your custom libraries(s).
 
            :param library_uid:  Library UID (optional)
-           :type library_uid: {str_type}
+           :type library_uid: str
 
            :param limit: limit number of fetched records (optional)
            :type limit: int
 
            :returns: metadata of libraries(s)
-           :rtype: dict
+           :rtype: dict (if uid is not None) or {"resources": [dict]} (if uid is None)
 
-           A way you might use me is:
+           **Example**:
 
             >>> library_details = client.runtimes.get_library_details(library_uid)
             >>> library_details = client.runtimes.get_library_details(library_uid=library_uid)
@@ -285,9 +288,9 @@ class Runtimes(WMLResource):
             :type details: dict
 
             :returns: runtime url
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**:
 
              >>> runtime_url = client.runtimes.get_url(runtime_details)
         """
@@ -315,9 +318,9 @@ class Runtimes(WMLResource):
             :type details: dict
 
             :returns: runtime uid
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**:
 
             >>> runtime_uid = client.runtimes.get_uid(runtime_details)
         """
@@ -345,9 +348,9 @@ class Runtimes(WMLResource):
             :type library_details: dict
 
             :returns: library url
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**:
 
              >>> library_url = client.runtimes.get_library_url(library_details)
         """
@@ -366,9 +369,9 @@ class Runtimes(WMLResource):
             :type library_details: dict
 
             :returns: library uid
-            :rtype: {str_type}
+            :rtype: str
 
-            A way you might use me is:
+            **Example**:
 
             >>> library_uid = client.runtimes.get_library_uid(library_details)
         """
@@ -400,12 +403,15 @@ class Runtimes(WMLResource):
             Delete runtime.
 
             :param runtime_uid: Runtime UID
-            :type runtime_uid: {str_type}
+            :type runtime_uid: str
 
             :param autoremove: if set to False, only runtime will be removed, if set to True, all libraries belonging only to it will be removed
             :type autoremove: bool
 
-            A way you might use me is:
+            :returns: returns the status message ("SUCCESS" or FAILED")
+            :rtype: str
+
+            **Example**:
 
             >>> client.runtimes.delete(runtime_uid)
             >>> client.runtimes.delete(runtime_uid, with_libraries=True)
@@ -422,9 +428,15 @@ class Runtimes(WMLResource):
 
         url = self._href_definitions.get_runtime_href(runtime_uid)
 
-        response_delete = requests.delete(
-            url,
-            headers=self._client._get_headers())
+        if not self._ICP:
+            response_delete = requests.delete(
+                url,
+                headers=self._client._get_headers())
+        else:
+            response_delete = requests.delete(
+                url,
+                headers=self._client._get_headers(),
+                verify=False)
 
         self._handle_response(204, u'runtime deletion', response_delete, False)
 
@@ -439,14 +451,16 @@ class Runtimes(WMLResource):
 
                 for uid in custom_libs_to_remove:
                     print('Deleting orphaned library \'{}\' during autoremove delete.'.format(uid))
-                    self.delete_library(uid)
+                    lib_delete_status = self.delete_library(uid)
+                    print(lib_delete_status)
+        return self._handle_response(204, u'runtime deletion', response_delete, False)
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def _delete_orphaned_libraries(self):
         """
             Delete all custom libraries without runtime.
 
-            A way you might use me is:
+            **Example**:
 
             >>> client.runtimes.delete_orphaned_libraries()
         """
@@ -457,7 +471,10 @@ class Runtimes(WMLResource):
             if len(self._get_runtimes_uids_for_lib(lib_uid, details)) == 0:
                 print('Deleting orphaned \'{}\' library... '.format(lib_uid), end="")
                 library_endpoint = self._href_definitions.get_custom_library_href(lib_uid)
-                response_delete = requests.delete(library_endpoint, headers=self._client._get_headers())
+                if not self._ICP:
+                    response_delete = requests.delete(library_endpoint, headers=self._client._get_headers())
+                else:
+                    response_delete = requests.delete(library_endpoint, headers=self._client._get_headers(), verify=False)
 
                 try:
                     self._handle_response(204, u'library deletion', response_delete, False)
@@ -472,17 +489,23 @@ class Runtimes(WMLResource):
             Delete custom library from repository.
 
             :param library_uid: stored custom library UID
-            :type library_uid: {str_type}
+            :type library_uid: str
 
-            A way you might use me is:
+            :returns: returns the status message ("SUCCESS" or FAILED")
+            :rtype: str
+
+            **Example**:
 
             >>> client.runtimes.delete_library(library_uid)
         """
         Runtimes._validate_type(library_uid, u'library_uid', STR_TYPE, True)
         library_endpoint = self._href_definitions.get_custom_library_href(library_uid)
-        response_delete = requests.delete(library_endpoint, headers=self._client._get_headers())
+        if not self._ICP:
+            response_delete = requests.delete(library_endpoint, headers=self._client._get_headers())
+        else:
+            response_delete = requests.delete(library_endpoint, headers=self._client._get_headers(), verify=False)
 
-        self._handle_response(204, u'library deletion', response_delete, False)
+        return self._handle_response(204, u'library deletion', response_delete, False)
 
     def list(self, limit=None):
         """
@@ -491,7 +514,13 @@ class Runtimes(WMLResource):
            :param limit: limit number of fetched records (optional)
            :type limit: int
 
-           A way you might use me is:
+           :returns: None
+           :rtype: None
+
+           .. note::
+               This function only prints the list of stored runtimes
+
+           **Example**:
 
            >>> client.runtimes.list()
         """
@@ -505,7 +534,7 @@ class Runtimes(WMLResource):
         """
            List runtimes uids for libraries.
 
-           A way you might use me is:
+           **Example**:
 
            >>> client.runtimes.list_runtimes_for_libraries()
            >>> client.runtimes.list_runtimes_for_libraries(library_uid)
@@ -532,12 +561,18 @@ class Runtimes(WMLResource):
            List custom libraries. If limit is set to None there will be only first 50 records shown.
 
            :param runtime_uid: Runtime UID
-           :type runtime_uid: {str_type}
+           :type runtime_uid: str
 
            :param limit: limit number of fetched records (optional)
            :type limit: int
 
-           A way you might use me is:
+           :returns: None
+           :rtype: None
+
+           .. note::
+               This function only prints the list of stored libraries
+
+           **Example**:
 
            >>> client.runtimes.list_libraries()
            >>> client.runtimes.list_libraries(runtime_uid)
@@ -577,14 +612,20 @@ class Runtimes(WMLResource):
             Downloads configuration file for runtime with specified UID.
 
             :param runtime_uid:  UID of runtime
-            :type runtime_uid: {str_type}
+            :type runtime_uid: str
             :param filename: filename of downloaded archive (optional)
-            :type filename: {str_type}
+            :type filename: str
 
-            :returns: path to downloaded file
-            :rtype: {str_type}
+            :returns: path to the downloaded file
+            :rtype: str
 
-            A way you might use me is:
+            .. note::
+                If filename is not specified, the default filename is "runtime_configuration.yaml".
+
+            Side effect:
+                save runtime configuration to file.
+
+            **Example**:
 
             >>> filename = client.runtimes.download_configuration(runtime_uid)
             >>> client.runtimes.download_configuration(runtime_uid, filename=filename)
@@ -598,9 +639,15 @@ class Runtimes(WMLResource):
 
         download_url = self._href_definitions.get_runtime_href(runtime_uid) + '/content'
 
-        response_get = requests.get(
-            download_url,
-            headers=self._client._get_headers())
+        if not self._ICP:
+            response_get = requests.get(
+                download_url,
+                headers=self._client._get_headers())
+        else:
+            response_get = requests.get(
+                download_url,
+                headers=self._client._get_headers(),
+                verify=False)
 
         if filename is None:
             filename = 'runtime_configuration.yaml'
@@ -610,10 +657,8 @@ class Runtimes(WMLResource):
                 new_file.write(response_get.content)
                 new_file.close()
 
-                print_text_header_h2(
-                    u'Successfully downloaded configuration file: ' + str(filename))
-
-                return filename
+                print(u'Successfully downloaded configuration file: ' + str(filename))
+                return os.getcwd()+"/"+filename
         else:
             raise WMLClientError(u'Unable to download configuration content: ' + response_get.text)
 
@@ -622,14 +667,21 @@ class Runtimes(WMLResource):
             Downloads custom library content with specified UID.
 
             :param library_uid:  UID of library
-            :type library_uid: {str_type}
+            :type library_uid: str
             :param filename: filename of downloaded archive (optional)
-            :type filename: {str_type}
+            :type filename: str
 
-            :returns: path to downloaded file
-            :rtype: {str_type}
+            :returns: path to the downloaded file
+            :rtype: str
 
-            A way you might use me is:
+            .. note::
+                If filename is not specified, the default filename is "<LIBRARY-NAME>-<LIBRARY-VERSION>.zip".
+
+
+            Side effect:
+                save library content to file.
+
+            **Example**:
 
             >>> filename = client.runtimes.download_library(library_uid)
             >>> client.runtimes.download_library(library_uid, filename=filename)
@@ -643,9 +695,15 @@ class Runtimes(WMLResource):
 
         download_url = self._href_definitions.get_custom_library_href(library_uid) + '/content'
 
-        response_get = requests.get(
-            download_url,
-            headers=self._client._get_headers())
+        if not self._ICP:
+            response_get = requests.get(
+                download_url,
+                headers=self._client._get_headers())
+        else:
+            response_get = requests.get(
+                download_url,
+                headers=self._client._get_headers(),
+                verify=False)
 
         if filename is None:
             details = self.get_library_details(library_uid)
@@ -656,9 +714,7 @@ class Runtimes(WMLResource):
                 new_file.write(response_get.content)
                 new_file.close()
 
-                print_text_header_h2(
-                    u'Successfully downloaded library content: ' + str(filename))
-
-                return filename
+                print(u'Successfully downloaded library content: ' + str(filename))
+                return os.getcwd()+"/"+filename
         else:
             raise WMLClientError(u'Unable to download library content: ' + response_get.text)
\ No newline at end of file
